<?php



require_once 'Classes/Model/Observation.php';



/**
 * @class ObservationSet
 *
 * This class represents an observation and a set of associated calibrations. These
 * calibrations will be within X minutes of the observation
 **/
class ObservationSet
{
  // PUBLIC

  public function __construct( ObservationSet $rhs = null )
  {
    $this->observation = null;
    $this->calibrations = null;
    $this->observation_data = null;
    $this->data_dir = Dir::ConvertPathString( '/pulsar/scratch01/data_archive/' );

    if( $rhs != null )
      $this->Copy( $rhs );
  }
  public function Copy( ObservationSet $rhs )
  {
    $this->observation = new Observation( $rhs->GetObservation() );

    $this->calibrations = Array();
    foreach( $rhs->calibrations as $next_cal )
      $this->calibrations[] = new Calibration( $next_cal );

    $this->observation_data = new ObservationData( $rhs->observation_data );
  }

  public function SetObservation( $s_observation ) { $this->observation = $s_observation; }
  public function GetObservation() { return $this->observation; }

  public function SetCalibrations( $s_calibrations ) { $this->calibrations = $s_calibrations; }
  public function GetCalibrations() { return $this->calibrations; }
  
  //! Get the adjusted filename, this will include scrunching parameters if set
  public function GetDataFile()
  {
    if( $this->observation_data != null )
      $file = new File( $this->observation_data->GetDataFile() );
    else
      $file = new File( $this->observation->GetFile() );
    
    return $file;
  }
  
  public function SetScrunchingParameters( ScrunchingParameters $s_scrunching_parameters ) 
  {
    $this->observation_data->SetScrunchingParameters( $s_scrunching_parameters );
  }
  
  public function GetScrunchingParameters() { return $this->observation_data->GetScrunchingParameters(); }
  
  //! This function creates the observation_data object and sets it up
  public function SetupObservationData()
  {
    $this->observation_data->SetPath( $this->observation->GetFile() );
    $this->observation_data->SetNBin( $this->observation->GetNBin() );
    $this->observation_data->SetNPol( $this->observation->GetNPol() );
    $this->observation_data->SetNChan( $this->observation->GetNChan() );
    $this->observation_data->SetNSub( $this->observation->GetNSub() );
    $this->observation_data->SetDataDir( $this->data_dir );
  }
  
  public function EstimateDownloadableSize()
  {
    $dl_size = $this->observation_data->EstimateDownloadableSize();
    foreach( $this->calibrations as $next_cal )
    {
      $dl_size += $next_cal->GetFile()->GetFileSize();
    }

    return $dl_size;
  }
  
  public function MakeDownloadables()
  {
    $this->observation_data->MakeDownloadableFile();
    foreach( $this->calibrations as $next_cal )
    {
      $src_file = $next_cal->GetFile();
      $dest_file = File::ConvertFullPath( $this->data_dir->GetPath() . '/' . $src_file->GetFilename() );
      if( !$dest_file->Exists() )
        link( $src_file->GetFullPath(), $dest_file->GetFullPath() );
    }
  }
  
  public function GetDownloadFiles()
  {
    $data_files = Array();
    $data_files[] = $this->observation_data->GetDataFile();
    foreach( $this->calibrations as $next_cal )
    {
      $cal_data_path = $this->data_dir->GetPath();
      $cal_data_path .= '/' . $next_cal->GetFile()->GetFilename();
      $data_files[] =  File::ConvertFullPath( $cal_data_path );
    }
    
    return $data_files;
  }

  // PRIVATE

  private $observation;
  private $calibrations;
  private $observation_data;
  private $data_dir;
};

