<?php



require_once 'Classes/General/File.php';
require_once GREEN_ZONE_DIR . 'Utils/StringUtils.php';
require_once 'Classes/Pulsar/ScrunchingParameters.php';



/**
 * This class represents an archive scrunching algorithm, though
 * the actual scrunching is performed by executing psrsh. You give
 * this algorithm
 * <ul>
 *  <li> the archive filename </li>
 *  <li> the directory to save the new scrunched file into </li>
 *  <li> the scrunching parameters </li>
 * </ul>
 **/

class ArchiveScruncher
{
  /// PUBLIC

    public function __construct()
    {
      $this->src_file = new File();  // This is a File class representing the source archive
      $this->dest_file = new File(); // This is a File class representing the destination archive
      $this->parameters = null;
    }

    public function SetSrcFile( File $s_src_file ) { $this->src_file = $s_src_file; }
    public function SetDestFile( File $s_dest_file ) { $this->dest_file = $s_dest_file; }
    
    public function SetParameters( $s_parameters ) { $this->parameters = $s_parameters; }
    public function GetParameters() { return $this->parameters; }

    /**
     * This is the algorithms execute method, determine the destination filename from the
     * source filename and scrunching parameters, create a script for psrsh to perform the
     * scrunching and to output the file, then exec psrsh with that script.
     **/

    public function Execute()
    {
      // TODO throw an exception if not scrunching needs to be done
      //       if( !$this->NeedsScrunching() )
      //       {
      //       }
      
//       $this->dest_file->SetFilename( $this->CreateScrunchedFilename( $this->src_file->GetFilename(name) ) );
    
      $script_file = $this->CreateScrunchingScript();
    
      $psrcmd = FIX_LD_LIB . PSRSH_CMD . " $script_file " . $this->src_file->GetFullPath();
//       echo 'psrcmd is ' . $psrcmd . ' <br> ';
    
      $output_array = Array();
      $retval = '';
      $last_line = exec( $psrcmd, $output_array, $retval );
    
      if( !file_exists( $this->dest_file->GetFullPath() ) )
	  throw new GZ_Exception( 'psrsh Failed to create scrunched file'.$this->dest_file->GetFullPath(),
				  'ArchiveScruncher::Execute' );
     }
    
   /**
    * Guess how much disk space the resulting scrunched file will take up
    **/

    public function GuessScrunchedFileSize()
    {

    }

  /// PRIVATE

    private $src_file;
    private $dest_file;
    private $parameters;


    /**
     * Return true if atleast one scrunching parameter is set, false otherwise.
     **/

    private function NeedsScrunching()
    {
      return ( $this->parameters->GetPScrunch() === false &&
	  $this->parameters->GetTScrunch() == '' &&
	  $this->parameters->GetFScrunch() == '' );
    }

    /**
     * Create a temporary file and write into it the instructions for psrsh
     * to perform to scrunch the file appropriately and write it out to the
     * target directory.
     **/

    private function CreateScrunchingScript()
    {
      if( $this->parameters->GetFScrunch() != '' )
	$psrscript .= "fscrunch_by $this->fscrunch \n";
      if( $this->parameters->GetTScrunch() != '' )
	$psrscript .= "tscrunch_by $this->tscrunch \n";
      if( $this->parameters->GetPScrunch() !== false )
	$psrscript .= "pscrunch \n";
      $psrscript .= 'unload ' . $this->dest_file->GetFullPath() . "\n";

      $script_filename = tempnam(TEMP_DIR, '');

    // TODO remove this chmod command, its for debugging only
	exec( 'chmod 666 ' . $script_filename );

      $outfile = fopen( $script_filename, 'r+' );
      // TODO throw an exception if we can't open the tempfile
      // if( $outfile === false )
      fwrite( $outfile, $psrscript );
      fclose( $outfile );

      return $script_filename;
    }
    
    
    /**
     * Call vap to determine the parameters for an archive, nsub,nchan,nbin,npol
     **/
    
    private function DetermineSubintParams( $filename, &$nsub, &$nchan, &$nbin, &$npol )
    {
      $cmd = FIX_LD_LIB . VAP_CMD . ' -c nbin,nchan,npol,nsub ' . $filename;
      $all_lines = Array();
      $retval = '';
      
      $last_line = exec( $cmd, $all_lines, $retval );
      
      $vap_response = eatwhite( $last_line );
      $nbin = strword( $vap_response, 1 );
      $nchan = strword( $vap_response, 2 );
      $npol = strword( $vap_response, 3 );
      $nsub = strword( $vap_response, 4 );
      
//       echo "$filename is (nsub,nchan,nbin,npol) ($nsub,$nchan,$nbin,$npol) <br>";
    } 
    



};


?>