<?php



require_once GREEN_ZONE_DIR . 'Utils/GZ_Exception.php';
require_once 'Classes/Pulsar/Archive.php';
require_once 'Classes/General/File.php';
require_once 'Classes/General/Dir.php';



/**
 * This class represents an algorithm to take a cspr2 observation, which is a directory
 * containing several archive files, and combining them into a single observation file.
 * It calls psradd to perform the actual work of combining the files.
 *
 * Usage
 *  $src_dir = new Dir( 'path to cpsr2 data directory' );
 *  $dest_file = new File();
 *  $dest_file->SetPath( 'path to file' );
 *  $dest_file->SetFielname( 'filename' );
 *  $combiner = new CPSR2Combiner();
 *  $combiner->SetSrcDir( $src_dir );
 *  $combiner->SetDestFile( $dest_file );
 *  $combiner->Execute();
 **/

class CPSR2Combiner
{
  // PUBLIC
  
  public function __construct()
  {
    $this->archives = Array();
    $this->dest_file = new File();
  }

  public function AddArchive( Archive $s_archive ) { $this->archives[] = $s_archive; }
  public function SetDestFile( $s_dest_file ) { $this->dest_file = $s_dest_file; }

  public function Execute()
  {
    $src_files_str = ' ';
    foreach( $this->archives as $next_archive )
          $src_files_str .= $next_archive->GetFullPath() . ' ';

    $psradd_cmd = FIX_LD_LIB . PSRADD_CMD . ' -f ' . $this->dest_file->GetFullPath() . $src_files_str;
    $retval = '';
    $all_lines = Array();

    exec( $psradd_cmd, $all_lines, $retval );

    // TODO throw an exception if the psradd call failed or if the file wasn't created
    if( !$this->dest_file->Exists() )
        throw new GZ_Exception( 'Failed to psradd files together', 'CPSR2Combiner::Execute' );
  }

  // PRIVATE

  private $src_dir;
  private $dest_file;
};


?>