<?php



require_once 'config.php';
require_once GREEN_ZONE_DIR . 'Forms/GZ_BaseForm.php';
require_once 'Forms/ObservationSetTable.php';
require_once 'Classes/Interfaces/ObsInterface.php';
require_once 'Classes/General/TarBuilder.php';



class CartForm extends GZ_BaseForm
{
  public function __construct( $s_gz )
  {
    parent::__construct( $s_gz );
    
    $this->LoadCart();
    
    $this->SetupObservationsTable();
    
    $this->SetupPaginator();
    
    $this->download_created = false;
  }
  

  
  public function Display()
  {
    $smarty = $this->gz->CreateSmarty();
    
    $this->gz->smarty_assign( $smarty, 'cart_size', count( $this->cart_contents ) );
    $smarty->display( 'CartFormHeader.tpl' );
    
    if( $this->download_created )
      echo 'Click <a href=downloads/random.tar> here </a> to download <br>';
    
    if( count( $this->cart_contents ) > 0 )
    {
      $this->paginator->Display();
      $this->obs_table->Display();
    }
    
    $smarty->display( 'CartFormFooter.tpl' );
  }
  
  public function ReadForm()
  {
    $this->obs_table->ReadForm();
    $this->paginator->ReadForm();
  }
  
  public function ProcessActions()
  {
    $next_form = '';
    
    $this->obs_table->ProcessActions();
    $this->paginator->ProcessActions();
    
    if( $this->ActionIs( 'BackToResults' ) )
      $next_form = 'QueryResultsForm';
    
    else if( $this->ActionIs( 'Download' ) )
    {
      $method = $this->GetPostString( 'DownloadMethod' );
      if( $method == 'RemoveFromCart' )
	$this->DeleteSelected();
      else if( $method == 'DownloadObsOnly' )
	$this->DownloadObservations();
      else if( $method == 'DownloadCalsOnly' )
	$this->DownloadCalibrations();
      else if( $method == 'DownloadObsAndCals' )
	$this->DownloadObsAndCals();
    }
    	  
    return $next_form;
  }
  
  // PRIVATE
      
  private $cart_contents;
  private $obs_table;
  private $paginator;
  private $download_created;
  
  private function SetupPaginator()
  {
    $table_pager = new GZ_Pager();
    $table_pager->SetTarget( $this->obs_table );
    $table_pager->SetItemsPerPage( 15 );
    
    $this->paginator = new GZ_Paginator( $this->gz, 'CartFormPaginator' );
    $this->paginator->AddPager( $table_pager );
  }
  
  private function SetupObservationsTable()
  {
    $this->obs_table = new ObservationSetTable( $this->gz, 'tabQueryResults' );
    
    $this->obs_table->SetColumnHeadings( Array( 'distance', 'filename', 'nbin', 'nchan', 'nsub', 'npol', 'src_name', 'project_id', 
					 'rajd', 'decjd', 'data_type', 'obsfreq', 'bw',
					 'scanlen', 'date', 'ut' ) );
    
    $this->obs_table->SetData( $this->cart_contents );
  }
  
  private function DownloadObservations()
  {
    $files_to_tar = Array();
    
    $selected_ids = $this->obs_table->GetSelectedIDs();
    
    foreach( $this->cart_contents as $next_item )
    {
      if( in_array( $next_item->GetObservation()->GetFileID(), $selected_ids ) )
      {
        $next_item->MakeDownloadables();
        $download_files = $next_item->GetDownloadFiles();
        foreach( $download_files as $next_file )
          $files_to_tar[] = $next_file;
      }
    }
    
    $tar_file = File::ConvertFullPath( '/pulsar/scratch01/data_archive/random.tar' );
    
    $tar_builder = new TarBuilder();
    $tar_builder->SetTarFile( $tar_file );
    foreach( $files_to_tar as $next_file )
      $tar_builder->AddFile( $next_file );
    $tar_builder->Execute();
    
    $web_link = File::ConvertFullPath( '/usr/local/work/pulsararchive/htdocs2/downloads/'. $tar_file->GetFilename() );
    
    echo 'will link ' . $tar_file->GetFullPath .' to ' . $web_link->GetFullPath() .'<br>';
    if( !$web_link->Exists() )
      link( $tar_file->GetFullPath(), $web_link->GetFullPath() );
    
    $this->download_created = true;
  }
  
  private function DownloadCalibrations()
  {
    echo 'downloaing calibrations <br> ';
  }
  
  private function DownloadObsAndCals()
  {
    echo 'downloading observations and cals <br> ';
  }
  
  private function DeleteSelected()
  {
    $selected_ids = $this->obs_table->GetSelectedIDs();
    if( count( $selected_ids ) > 0 )
    {
      $new_cart_contents = Array();
      foreach( $this->cart_contents as $next_cart_item )
      {
	if( !in_array( $next_cart_item->GetObservation()->GetFileID(), $selected_ids ) )
	   $new_cart_contents[] = $next_cart_item;
      }
      $this->cart_contents = $new_cart_contents;
      $this->SaveCart();
      $this->obs_table->SetData( $new_cart_contents );
      $this->paginator->Paginate();
    }
  }
  
  private function SaveCart()
  {
    global $_SESSION;
    
    $_SESSION['cart_contents'] = $this->cart_contents;
  }
  
  private function LoadCart()
  {
    global $_SESSION;
    
    $this->cart_contents = Array();
    if( isset( $_SESSION['cart_contents'] ) )
      $this->cart_contents = $_SESSION['cart_contents'];
  }
  
};


?>